@@ -1,5 +1,7 @@
+import functools
 import re
 import time
+import sys
 from .meters import Counter, Histogram, Meter, Timer, Gauge, CallbackGauge, SimpleGauge
 
 
@@ -27,23 +29,23 @@ class MetricsRegistry(object):
         Use this method to manually add custom metric instances to the registry
         which are not created with their constructor's default arguments,
         e.g. Histograms with a different size.
-        
+
         :param key: name of the metric
         :type key: C{str}
         :param metric: instance of Histogram, Meter, Gauge, Timer or Counter
         """
-        class_map = { 
-           Histogram: self._histograms,
-           Meter: self._meters,
-           Gauge: self.gauges,
-           Timer: self._timers,
-           Counter: self._counters,
-        }
-        for cls, registry in class_map.iteritems():
+        class_map = (
+           (Histogram, self._histograms),
+           (Meter, self._meters),
+           (Gauge, self._gauges),
+           (Timer, self._timers),
+           (Counter, self._counters),
+        )
+        for cls, registry in class_map:
             if isinstance(metric, cls):
                 if key in registry:
                     raise LookupError("Metric %r already registered" % key)
-                registry[key] = registry
+                registry[key] = metric
                 return
         raise TypeError("Invalid class. Could not register metric %r" % key)
 
@@ -98,6 +100,9 @@ class MetricsRegistry(object):
             self._meters[key] = Meter(clock=self._clock)
         return self._meters[key]
 
+    def create_sink(self):
+        return None
+
     def timer(self, key):
         """
         Gets a timer based on a key, creates a new one if it does not exist.
@@ -108,7 +113,7 @@ class MetricsRegistry(object):
         :return: L{Timer}
         """
         if key not in self._timers:
-            self._timers[key] = Timer(clock=self._clock)
+            self._timers[key] = Timer(clock=self._clock, sink=self.create_sink())
         return self._timers[key]
 
     def clear(self):
@@ -162,6 +167,7 @@ class MetricsRegistry(object):
             timer = self._timers[key]
             snapshot = timer.get_snapshot()
             res = {"avg": timer.get_mean(),
+                   "sum": timer.get_sum(),
                    "count": timer.get_count(),
                    "max": timer.get_max(),
                    "min": timer.get_min(),
@@ -170,6 +176,7 @@ class MetricsRegistry(object):
                    "5m_rate": timer.get_five_minute_rate(),
                    "1m_rate": timer.get_one_minute_rate(),
                    "mean_rate": timer.get_mean_rate(),
+                   "50_percentile": snapshot.get_median(),
                    "75_percentile": snapshot.get_75th_percentile(),
                    "95_percentile": snapshot.get_95th_percentile(),
                    "99_percentile": snapshot.get_99th_percentile(),
@@ -243,8 +250,8 @@ class RegexRegistry(MetricsRegistry):
     def counter(self, key):
         return super(RegexRegistry, self).counter(self._get_key(key))
 
-    def gauge(self, key, gauge=None):
-        return super(RegexRegistry, self).gauge(self._get_key(key), gauge)
+    def gauge(self, key, gauge=None, default=float("nan")):
+        return super(RegexRegistry, self).gauge(self._get_key(key), gauge, default)
 
     def meter(self, key):
         return super(RegexRegistry, self).meter(self._get_key(key))
@@ -289,6 +296,10 @@ def dump_metrics():
 def clear():
     return _global_registry.clear()
 
+def get_qualname(obj):
+    if sys.version_info[0] > 2:
+        return obj.__qualname__
+    return obj.__name__
 
 def count_calls(fn):
     """
@@ -300,12 +311,10 @@ def count_calls(fn):
     :return: the decorated function
     :rtype: C{func}
     """
-    def wrapper(*args):
-        counter("%s_calls" % fn.__name__).inc()
-        try:
-            fn(*args)
-        except:
-            raise
+    @functools.wraps(fn)
+    def wrapper(*args, **kwargs):
+        counter("%s_calls" % get_qualname(fn)).inc()
+        return fn(*args, **kwargs)
     return wrapper
 
 
@@ -319,12 +328,10 @@ def meter_calls(fn):
     :return: the decorated function
     :rtype: C{func}
     """
-    def wrapper(*args):
-        meter("%s_calls" % fn.__name__).mark()
-        try:
-            fn(*args)
-        except:
-            raise
+    @functools.wraps(fn)
+    def wrapper(*args, **kwargs):
+        meter("%s_calls" % get_qualname(fn)).mark()
+        return fn(*args, **kwargs)
     return wrapper
 
 
@@ -338,14 +345,13 @@ def hist_calls(fn):
     :return: the decorated function
     :rtype: C{func}
     """
-    def wrapper(*args):
-        _histogram = histogram("%s_calls" % fn.__name__)
-        try:
-            rtn = fn(*args)
-            if type(rtn) in (int, float):
-                _histogram.update(rtn)
-        except:
-            raise
+    @functools.wraps(fn)
+    def wrapper(*args, **kwargs):
+        _histogram = histogram("%s_calls" % get_qualname(fn))
+        rtn = fn(*args, **kwargs)
+        if type(rtn) in (int, float):
+            _histogram.update(rtn)
+        return rtn
     return wrapper
 
 
@@ -359,7 +365,10 @@ def time_calls(fn):
     :return: the decorated function
     :rtype: C{func}
     """
-    def wrapper(*args):
-        _timer = timer("%s_calls" % fn.__name__)
-        with _timer.time(fn = fn.__name__):
-            fn(*args)
+    @functools.wraps(fn)
+    def wrapper(*args, **kwargs):
+        _timer = timer("%s_calls" % get_qualname(fn))
+        with _timer.time(fn = get_qualname(fn)):
+            return fn(*args, **kwargs)
+    return wrapper
+
